package esercizio2;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

public class GestoreEsami {

	private ArrayList<Esame> esami;
	private HashMap<String,ArrayList<Esame>> studenteEsami;
	private HashMap<String,ArrayList<Esame>> esameStudenti;
	
	public GestoreEsami() {
		esami = new ArrayList<Esame>();
		studenteEsami = new HashMap<String, ArrayList<Esame>>();
		esameStudenti = new HashMap<String, ArrayList<Esame>>();
	}
	
	private void aggiungiEsame(String matricola, String nome, String cognome, Integer voto, String nomeEsame) {
		Esame e = new Esame(matricola, cognome, nome, voto, nomeEsame);
		esami.add(e);
		if(!studenteEsami.containsKey(matricola))
			studenteEsami.put(matricola, new ArrayList<Esame>());
		studenteEsami.get(matricola).add(e);
		if(!esameStudenti.containsKey(nomeEsame))
			esameStudenti.put(nomeEsame, new ArrayList<Esame>());
		esameStudenti.get(nomeEsame).add(e);
	}
	
	void importaEsami(String filename) {
		try {
			BufferedReader b = new BufferedReader(new FileReader(filename));
			while (b.ready()) {
				String matricola = b.readLine();
				String nome = b.readLine();
				String cognome = b.readLine();
				Integer voto = Integer.parseInt(b.readLine());
				String nomeEsame = b.readLine();
				aggiungiEsame(matricola, cognome, nome, voto, nomeEsame);
			}
			b.close();
		} catch (IOException e) {
			System.out.println("Errore nella lettura del file " + filename);
		}		
	}
	
	void esportaEsami(String filename) {
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter(new FileWriter(filename));
			for (int i = 0; i < esami.size()-1; i++) {
				writer.append(esami.get(i).toString());		
				writer.newLine();
			}
     	    // Non mettiamo il fine linea per l'ultimo esame
			writer.append(esami.get(esami.size()-1).toString());
			writer.close();
		} catch (IOException e) {
			System.out.println("Errore nella gestione del file " + filename);
		}
	}
	
	void inserisciEsame() {
		Scanner in = new Scanner(System.in);
		System.out.println("Matricola: ");
		String matricola = in.nextLine();
		System.out.println("Nome: ");
		String nome = in.nextLine();
		System.out.println("Cognome: ");
		String cognome = in.nextLine();
		System.out.println("Voto: ");
		Integer voto = Integer.parseInt(in.nextLine());
		System.out.println("Nome esame:");
		String nomeEsame = in.nextLine();
		System.out.println("Esame inserito");
		aggiungiEsame(matricola, cognome, nome, voto, nomeEsame);
	}
	
	public void calcolaMediaStudente(String matricola) {
		Double media = mediaStudente(matricola);
		if(media == null)
			System.out.println("Lo studente con matricola " + matricola + " non ha sostenuto esami");			
		else
			System.out.println("La media dello studente con matricola " + matricola + " è: " + media);			
	}
	
	private Double mediaStudente(String matricola) {
		if(!studenteEsami.containsKey(matricola))
			return null;
		ArrayList<Esame> esamiStudente = studenteEsami.get(matricola);		
		double somma = 0.0;
		for(Esame e : esamiStudente) {
			somma += e.getVoto();
		}
		return somma / esamiStudente.size();
	}
	
	private Double mediaStudenteAggiungendoEsame(String matricola) {
		if(!studenteEsami.containsKey(matricola))
			return null;
		ArrayList<Esame> esamiStudente = studenteEsami.get(matricola);		
		double somma = 30.0;
		for(Esame e : esamiStudente) {
			somma += e.getVoto();
		}		
		return somma / (esamiStudente.size()+1);
	}
	
	public void numeroStudentiEsame(String nomeEsame) {
		System.out.print("L'esame " + nomeEsame + " è stato superato da ");		
		if(!esameStudenti.containsKey(nomeEsame))
			System.out.println("0 studenti");
		else
			System.out.println(esameStudenti.get(nomeEsame).size() + " studenti");
	}
	
	public void numeroEsamiStudente(String matricola) {
		System.out.print("Lo studente con matricola " + matricola + " ha superato ");
		if(!studenteEsami.containsKey(matricola))
			System.out.println("0 esami");
		else
			System.out.println(studenteEsami.get(matricola).size() + " esami");
	}
	
	private int numeroEsamiConVotoSuperioreUguale(String matricola, int voto) {
		ArrayList<Esame> esamiStudente = studenteEsami.get(matricola);
		int count = 0;
		for(Esame e : esamiStudente) {
			if(e.getVoto() >= voto)
				count++;
		}
		return count;
	}
	
	public void calcolaStudentiBorsaDiStudio() {
		System.out.println("Le matricole degli studenti che hanno diritto alla borsa di studio:");
		for(String matricola : studenteEsami.keySet()) {
			int num = numeroEsamiConVotoSuperioreUguale(matricola, 26);
			if(num >= 5 && mediaStudente(matricola) >= 28.0)
				System.out.println(matricola);
		}
	}
	
	public void borsaDiStudioPossibile(String matricola) {
		int num = numeroEsamiConVotoSuperioreUguale(matricola, 26);
		if(num >= 4 && mediaStudenteAggiungendoEsame(matricola) >= 28.0)
			System.out.println("Lo studente con matricola " + matricola + " può ambire alla borsa di studio");
		else
			System.out.println("Lo studente con matricola " + matricola + " non può ambire alla borsa di studio");
	}
	
	public static void main(String[] args) {
		GestoreEsami g = new GestoreEsami();
		g.importaEsami("esami.txt");
		g.esportaEsami("esamicopia.txt");
		g.calcolaMediaStudente("1");
		g.calcolaStudentiBorsaDiStudio();
		g.borsaDiStudioPossibile("1");
		g.numeroEsamiStudente("1");
		g.numeroStudentiEsame("Fondamenti di programmazione");
	}
}
